/**
 * Drag and drop example. This program is written for the people on the GP32x Xtreme
 * boards as a reply on a question how to drag stuff around. Some of the code may look
 * confusing at first, the basic knowledge(setting up etc) is noted rather compact, but
 * the main mechanic should be clear from the code and comments. If not, you know where 
 * to find me :) - yell gp32dev chat/boards/msn
 *
 * Author: Moogle or Roelf Leenders
 * Date: 7 April 2005
 * Contact: MSN - leenders_roelf@hotmail.com Mail - roelf.leenders@gmail.com
 *
 */
 
program demonstration_dragdrop;

global
  //Declare an array holding all the characters we'll be needing to drag around
  String example_text[8] = 'G','P','3','2','F','e','n','i','x';

  //Some helping variables
  graphic1,graphic2;

  //holds relative coordinates and ID of the process we're dragging
  relativeX,relativeY,dragID;


begin

//Initialize the screen to 16 bits normal GP32 resolution:
full_screen = false;
set_title("Drag&Drop - Moogle@Gp32Xtreme"
graph_mode = mode_16bits;
set_mode(m320x240);

//Change the text colour and background colour to something funkier than plain black and white
set_text_color(rgb(150,150,150));
//Background can always be accessed as map number zero from fpg number zero
map_clear(0,0,rgb(0,0,100));


//write_int(0,0,0,0,&dragID);

/*
  Generate the graphics/ processes holding them
  Not that interesting, I use the graphics generation mainly because it's easier when you want
  to send someone the code to just send the .prg, then needing to bother with other files too.
  The generation code looks quite complicated on first sight and is not really mend for those
  that want to learn how to use the drag and drop principle, so I wont document it that heavily.
  Main idea for creating the mousepointer is to setup a few criteria a point needs to fall into
  to be part of the arrow, otherwise it'll be black. Check it out if you want.
*/

//mouse:
mouse.graph = new_map(30,30,16);
set_center(0,mouse.graph,0,0);

for(x=0;x<30;x++)
  for(y=0;y<30;y++)
    if( (x+y =< 30 and abs(fget_angle(0,0,x,y)-fget_angle(0,0,30,30)) < 25000 ) or (abs(x-y) < 4 and x+y > 20))
      map_put_pixel(0,mouse.graph,x,y,rgb(254,254,254));
    end
  end
end

//Since 30x30 is quite big for a mouse on this resolution(320x240), display it at 50% of it's size
mouse.size = 50;

/*
  The code below creates a graphic of 1 by 1 pixel, to serve as mousepointer. In cases where
  you want a mouse pointer and you are planning on using collision commands with it, it's often better
  to use two graphics, one of the mouse pointer(typically an arrow), and under that a 1x1 graphic
  to check for collisions with. Doing this avoids the bug that you can drag around things with
  the tail of the mouse, and not only with the 'hotspot'. Here I'm using the arrow as mouse graphic
  (ie, in the mouse struct) because that's harder to check for collisions with(mouse.bla has no own
  process where you can put your collision commands in). the main process has the small graphic re-
  presenting the 'hotspot'. The graphic is coloured white because transparent pixels do not collide.
*/
graph = new_map(1,1,16);
map_clear(0,graph,rgb(254,254,254));


/*
  This is where the graphics with the letters get made and the letter processes themself get created.
  I use a for loop to generate 9 letter processes(x goes from 0 to 8, both included). For text I use
  the array filled with letters declared in the globals section at the top.
*/
for(x=0;x<9;x++)

  //Create a new graphic
  graphic1 = new_map(25,25,16);

  //paint it dark grey(not black, that would be transparent)
  map_clear(0,graphic1,rgb(20,20,20,));

  //Create another graphic, black with the current letter(held in the
  //array on position x) in it.
  graphic2 = write_in_map(0,example_text[x],4);

  //Pastes the black graphic over the dark grey one, and hereby losing
  //the transparency leaving a dark grey square with a letter on it
  map_put(0,graphic1,graphic2,12,12);

  //Create a new process with this graphic, on a calculated position on screen.
  letter(160+(x-4)*30,120,graphic1);

end

//Main loop:
loop
  //Copy the mouse coordinates(since that's where we want the hotspot to be)
  x = mouse.x;
  y = mouse.y;

  //On left mouseclick, go go go!
  if(mouse.left)
    //Fetch a colliding letter(or not if there are none):
    dragID = collision(type letter);

    //But if there are any(checking with exists), save the coordinate position
    //relative to the mouse coordinates(in this case, this process has that same coordinates)
    if(exists(dragID))
      //As said, the relative positions
      relativeX = mouse.x - dragID.x;
      relativeY = mouse.y - dragID.y;
    end

    //Now we need to drag it around while the the player keeps down the left mouse button
    while(mouse.left)
      
      //And while so, we need not forget the demands of the normal loop, ie, keep placing
      //the hotspot(our process graphic) on the mouse coordinates.
      x = mouse.x;
      y = mouse.y;

      //Here's the crux, if a process was found colliding at the time of the click, he needs
      //to be moved every frame for as long as the mouse button stays pressed. This means, that
      //the coordinates of the process should move the exact same amounts the mouse does, and so
      //the relative coordinates stay the same. We saved the relative position, and so we can back
      //track where the process should be. This is kind of the point of the whole program :)
      if(exists(dragID))
        dragID.x = mouse.x - relativeX;
        dragID.y = mouse.y - relativeY;
      end

      //And another demand of the normal loop, the escape key. done!
      //Quit on Escape key
      if(key(_esc))exit("",0);end
      //And of course a good old frame, since we stay in here quite some time,
      //even more if we don't put it in here ;)
      frame;
    end
  end

  //Quit on Escape key
  if(key(_esc))exit("",0);end
  frame;
end

end

/**
 * Process holding a specified graphic on a specific position. Not really much
 * to see here :)
 *
 */
process letter(x,y,graph)

begin

  loop
    frame;
  end

end
